import QtQuick 2.2
import QtQuick.Controls 1.2
import QtQuick.Layouts 1.1
import QtPositioning 5.2
import cz.mlich 1.0
import "Theme.js" as Theme

ApplicationWindow {
    id: app
    visible: true
    width: Theme.screen_width
    height: Theme.screen_height
    title: "Letiště"

    InfoPanel {
        id: infoPanel;
        anchors.bottom: parent.bottom;
        anchors.left: parent.left
        anchors.right: parent.rigth;
        width: parent.width;
        height: Theme.info_panel_height

    }

    property bool sideBarVisible: (searchBtn.checked || navBtn.checked || settingsBtn.checked || mapObjectsBtn.checked);
    property bool landscape: app.width > app.height;

    Rectangle {
        id: mapRectangle
        anchors.top: parent.top;
        anchors.bottom: infoPanel.top;
        anchors.left: parent.left;
        width:  sideBarVisible ?  0.6*app.width : app.width
        clip: true;


        PinchMap {
            id: pinchMap;
            anchors.fill: parent;
            onClickedIntoMap: {
                mapObjectsBtn.checked = true;
            }
            onPannedManually: {
                trackButton.checked = false;
            }

            rotationEnabled: settingsDialog.rotationEnabled
        }

    }

    Rectangle {
        id: sidebarDialogs
        anchors.left: mapRectangle.right
        anchors.right: parent.right;
        anchors.top: parent.top;
        anchors.bottom: infoPanel.top;

        SearchDialog {
            id: searchDialog;
            visible: searchBtn.checked;
            anchors.fill: parent;
            airfieldsModel: pinchMap.airportModelA
            navModel: pinchMap.navigationModelA;
            onShowNav: {
                navBtn.checked = true;
            }

        }

        NavDialog {
            id: navDialog
            anchors.fill: parent;
            visible: navBtn.checked;
            navigationModel: pinchMap.navigationModelA;
        }

        SettingsDialog {
            id: settingsDialog;
            anchors.fill: parent;
            visible: settingsBtn.checked;
            onReloadNotams: {
                download_notam();
            }
        }


        MapClickedDialog {
            anchors.fill: parent;
            visible: mapObjectsBtn.checked;

            clickedModel: pinchMap.clickedModelA;
            navModel: pinchMap.navigationModelA;
            onShowNav: {
                navBtn.checked = true;
            }
            onSelectedAirspace: {
                pinchMap.selectedAirspace = name;
                pinchMap.requestUpdate();
            }
            onVisibleChanged: {
                pinchMap.selectedAirspace = "";
            }
        }

    }

    ExclusiveGroup {
        id: dialogsExclusiveGroup;
    }

    Column {
        id: imageButtonsColumn

        anchors.top:parent.top;
//        anchors.topMargin: Theme.margin
//        spacing: Theme.margin
        ImageButton {
            id: settingsBtn
            name: "settings"
            exclusiveGroup: dialogsExclusiveGroup;
        }
        ImageButton {
            id: navBtn;
            name: "nav"
            exclusiveGroup: dialogsExclusiveGroup;
        }
        ImageButton {
            id: searchBtn
            name: "search"
            exclusiveGroup: dialogsExclusiveGroup;
        }
        ImageButton {
            id: trackButton;
            name: "track"
            onClicked: {
                pinchMap.setNavLatLon(positionSource.position.coordinate.latitude, positionSource.position.coordinate.longitude)
            }
        }

        ImageButton {
            id: refreshButton
            name: "refresh"
            onClicked: {
                download_notam();
            }
        }

        ImageButton {
            visible: false; // this is just hidden
            id: mapObjectsBtn;
            exclusiveGroup: dialogsExclusiveGroup;
            //            name: "map"
        }
    }

    PositionSource {

        nmeaSource: "file:///home/jmlich/workspace/dnotam/GPS_20130501_121202.log"

        id: positionSource
        updateInterval: 1000
        active: true;

        onPositionChanged: {
            //            if (position.latitudeValid && ((new Date().getTime()-position.timestamp.getTime()) < 60000)) { // position could be valid but very old

            var coord = position.coordinate;

            if (position.directionValid) {
                infoPanel.direction = position.direction
                pinchMap.positionIndicator.azimuth = position.direction
            }
            if (position.speedValid) {
                infoPanel.speed = position.speed
                pinchMap.positionIndicator.speed = position.speed;
            }
            if (position.altitudeValid) {
                infoPanel.altitude = coord.altitude
            }

            if (trackButton.checked && position.latitudeValid ) {
                pinchMap.setNavLatLon(coord.latitude, coord.longitude)

                if (position.directionValid) {
                    pinchMap.angle = -position.direction;
                }
            }

            if (position.latitudeValid) {
                navDialog.currentLatitude = coord.latitude
                navDialog.currentLongitude = coord.longitude;
            }
            navDialog.currentPositionValid = position.latitudeValid;

            pinchMap.positionIndicator.latitude  = coord.latitude;
            pinchMap.positionIndicator.longitude = coord.longitude;
            pinchMap.positionIndicator.positionValid = position.latitudeValid

        }
    }

    Component.onCompleted: {
        download();
        download_notam();
    }

    function download() {
        var http = new XMLHttpRequest()
        var url = "http://pcmlich.fit.vutbr.cz/dnotam/regions.php?version="+ encodeURIComponent(APP_VERSION);
        console.log(url)
        http.open("GET", url, true);
        http.onreadystatechange = function() {
            if (http.readyState === XMLHttpRequest.DONE) {
                if (http.status === 200) {
                    try {
                        var obj = JSON.parse(http.responseText)
                        if (obj.message !== undefined) {
                            showError(arr.message)
                        }

                        var arr = obj.regions;

                        for (var i = 0; i < arr.length; i++) {
                            var item = arr[i];
                            item.download_status = "unknown";
                            settingsDialog.regionsModel.append(arr[i])
                        }

                    } catch (exception) {
                        showError(url + "\n" + exception.message)
                    }

                } else {
                    showError(url + "\n" + http.status + " " + http.statusText)
                }
            }
        }
        http.send();
    }

    function download_notam() {
        var http = new XMLHttpRequest()
        var url = "http://pcmlich.fit.vutbr.cz/dnotam/?json&version="+ encodeURIComponent(APP_VERSION);

        var pipe = [];
        pipe.push({"lat": positionSource.position.coordinate.latitude, "lon":positionSource.position.coordinate.longitude})

        for (var i = 0; i < pinchMap.navigationModelA.count; i++) {
            var item = pinchMap.navigationModelA.get(i);
            pipe.push({"lat": item.lat, "lon": item.lon})
        }

        url = url + "&bts="+settingsDialog.timestampBeginPosition;
        url = url + "&ets="+settingsDialog.timestampEndPosition;

        if (pinchMap.navigationModelA.count > 0) {
            url = url + "&pipe="+encodeURIComponent(JSON.stringify(pipe))
        }

        http.open("GET", url, true);
        http.onreadystatechange = function() {
            if (http.readyState === XMLHttpRequest.DONE) {
                if (http.status === 200) {
                    try {
                        pinchMap.air = JSON.parse(http.responseText);
                        pinchMap.requestUpdate();
                        if (pinchMap.air.message !== undefined) {
                            showError(pinchMap.air.message)

                        }
                    } catch (exception) {
                        showError(url + "\n" + exception.message)
                    }
                } else {
                    showError(url + "\n" + http.status + " " + http.statusText)
                }
                refreshButton.secondImage = false; // progress indicator
            }
        }
        refreshButton.secondImage = true; // progress indicator
        http.send();

    }

    function showError(message) {
        console.log(message);
        errorDialog.text = message;
        errorDialog.visible = true;
    }


    Rectangle {
        id: errorDialog
        anchors.fill: parent;
        color:  Theme.error_dialog_background_color;
        visible: false;
        property alias text: errorText.text
        Flickable {
            anchors.fill: parent;
            anchors.topMargin: Theme.margin
            anchors.bottomMargin: Theme.margin
            anchors.leftMargin: Theme.margin*10
            anchors.rightMargin: Theme.margin*10
            contentHeight: errorDialogColumn.height;
            Column {
                id: errorDialogColumn
                anchors.left: parent.left;
                anchors.right: parent.right;
                spacing: Theme.margin

                Text {
                    color: Theme.header_color;
                    font.pixelSize: Theme.header_pixelSize
                    anchors.left: parent.left;
                    anchors.right: parent.right;
                    horizontalAlignment: Text.AlignHCenter
                    text: "Error"
                }

                Text {
                    id: errorText
                    color: Theme.primary_color;
                    font.pixelSize: Theme.primary_pixelSize
//                    horizontalAlignment: Text.AlignHCenter
                    wrapMode: Text.Wrap
                    anchors.left: parent.left;
                    anchors.right: parent.right;
                }
            }

        }
        MouseArea {
            anchors.fill: parent;
            onClicked: errorDialog.visible = false;
        }



    }

    Timer {
        interval: 1000;
        running: true;
        repeat: true;
        onTriggered: {

            if (pinchMap.warningModelA.count == 0) {
                return;
            }

            var str = "";

            for (var i = 0; i < pinchMap.warningModelA.count; i++) {
                var item = pinchMap.warningModelA.get(i);
                if (item.distance > 0) {
                    str += item.designator + "; "
                }
            }

            infoPanel.warningText = (str.length > 0) ? ("Blížíte se k " + str) : "";

        }
    }

}
